"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const axios_1 = require("axios");
class File {
    constructor(_rootRef, _fileName, _content) {
        this._rootRef = _rootRef;
        this._fileName = _fileName;
        this._content = _content;
        this._hasUpdates = true;
    }
    get hasUpdates() {
        return this._hasUpdates;
    }
    get name() {
        return this._fileName;
    }
    getContent() {
        return this._content;
    }
    overwrite(newContent) {
        this._content = newContent;
        this._hasUpdates = true;
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = `${constants_1.default.githubGists}/${this._rootRef.gistId}`;
            const body = {
                public: this._rootRef.isPublic,
                files: {
                    [this._fileName]: {
                        content: this._content
                    }
                }
            };
            try {
                yield axios_1.default.post(url, body, this._rootRef._authConfig());
            }
            catch (e) {
                throw new Error('Couldn\'t save file: ' + this._fileName);
            }
            this._hasUpdates = false;
        });
    }
    fetchLatest() {
        return __awaiter(this, void 0, void 0, function* () {
            const latestCommit = yield this._rootRef._getLatestCommit();
            const url = this._latestFetchUrl(latestCommit);
            try {
                this._content = yield axios_1.default.get(url, this._rootRef._authConfig());
            }
            catch (e) {
                throw new Error('Couldn\'t fetch latest data of ' + this._fileName);
            }
            this._hasUpdates = false;
        });
    }
    _latestFetchUrl(commitId) {
        const { allowCors, username, gistId, } = this._rootRef;
        const corsPrefix = allowCors ? constants_1.default.corsAnywhere : '';
        return `${corsPrefix}` +
            `https://gist.githubusercontent.com/${username}` +
            `/${gistId}/raw/${commitId}/${this._fileName}`;
    }
}
exports.default = File;
